@@ -1,9 +1,9 @@
 /*
  *  Boa, an http server
- *  Copyright (C) 1995 Paul Phillips <psp@well.com>
- *  Some changes Copyright (C) 1996 Larry Doolittle <ldoolitt@jlab.org>
- *  Some changes Copyright (C) 1996-99 Jon Nelson <jnelson@boa.org>
+ *  Copyright (C) 1995 Paul Phillips <paulp@go2net.com>
+ *  Some changes Copyright (C) 1996 Larry Doolittle <ldoolitt@boa.org>
  *  Some changes Copyright (C) 1996 Russ Nelson <nelson@crynwr.com>
+ *  Some changes Copyright (C) 1996-2002 Jon Nelson <jnelson@boa.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,13 +21,13 @@
  *
  */
 
-/* $Id: alias.c,v 1.55 2001/07/18 03:39:11 jnelson Exp $ */
+/* $Id: alias.c,v 1.70.2.4 2002/07/28 02:46:52 jnelson Exp $ */
 
 #include "boa.h"
 
 static alias *alias_hashtable[ALIAS_HASHTABLE_SIZE];
 
-inline int get_alias_hash_value(char *file);
+int get_alias_hash_value(char *file);
 
 /*
  * Name: get_alias_hash_value
@@ -37,7 +37,7 @@ inline int get_alias_hash_value(char *file);
  * Note: stops at first '/' (or '\0')
  */
 
-inline int get_alias_hash_value(char *file)
+int get_alias_hash_value(char *file)
 {
     unsigned int hash = 0;
     unsigned int index = 0;
@@ -61,22 +61,34 @@ void add_alias(char *fakename, char *realname, int type)
 {
     int hash;
     alias *old, *new;
+    int fakelen, reallen;
+
+    /* sanity checking */
+    if (fakename == NULL || realname == NULL) {
+        DIE("NULL values sent to add_alias");
+    }
+
+    fakelen = strlen(fakename);
+    reallen = strlen(realname);
+    if (fakelen == 0 || reallen == 0) {
+        DIE("empty values sent to add_alias");
+    }
 
     hash = get_alias_hash_value(fakename);
 
     old = alias_hashtable[hash];
 
-    if (old)
+    if (old) {
         while (old->next) {
             if (!strcmp(fakename, old->fakename)) /* don't add twice */
                 return;
             old = old->next;
         }
+    }
+
     new = (alias *) malloc(sizeof (alias));
     if (!new) {
-        log_error_mesg(__FILE__, __LINE__,
-                       "out of memory adding alias to hash");
-        exit(errno);
+        DIE("out of memory adding alias to hash");
     }
 
     if (old)
@@ -85,9 +97,25 @@ void add_alias(char *fakename, char *realname, int type)
         alias_hashtable[hash] = new;
 
     new->fakename = strdup(fakename);
-    new->fake_len = strlen(fakename);
-    new->realname = strdup(realname);
-    new->real_len = strlen(realname);
+    if (!new->fakename) {
+        DIE("failed strdup");
+    }
+    new->fake_len = fakelen;
+    /* check for "here" */
+    if (realname[0] == '.' && realname[1] == '/') {
+        new->realname = normalize_path(realname+2);
+        if (!new->realname) {
+            /* superfluous - normalize_path checks for NULL return values. */
+            DIE("normalize_path returned NULL");
+        }
+        reallen = strlen(new->realname);
+    } else {
+        new->realname = strdup(realname);
+        if (!new->realname) {
+            DIE("strdup of realname failed");
+        }
+    }
+    new->real_len = reallen;
 
     new->type = type;
     new->next = NULL;
@@ -103,13 +131,15 @@ void add_alias(char *fakename, char *realname, int type)
  * alias structure or NULL if not found
  */
 
-alias *find_alias(char *uri)
+alias *find_alias(char *uri, int urilen)
 {
     alias *current;
     int hash;
 
     /* Find ScriptAlias, Alias, or Redirect */
 
+    if (urilen == 0)
+        urilen = strlen(uri);
     hash = get_alias_hash_value(uri);
 
     current = alias_hashtable[hash];
@@ -119,16 +149,30 @@ alias *find_alias(char *uri)
                 "%s:%d - comparing \"%s\" (request) to \"%s\" (alias): ",
                 __FILE__, __LINE__, uri, current->fakename);
 #endif
-        if (!memcmp(uri, current->fakename, current->fake_len)) {
+        /* current->fake_len must always be:
+         *  shorter or equal to the uri
+         */
+        /*
+         * when performing matches:
+         * If the virtual part of the url ends in '/', and
+         * we get a match, stop there.
+         * Otherwise, we require '/' or '\0' at the end of the url.
+         * We only check if the virtual path does *not* end in '/'
+         */
+        if (current->fake_len <= urilen &&
+            !memcmp(uri, current->fakename, current->fake_len) &&
+            (current->fakename[current->fake_len - 1] == '/' ||
+             (current->fakename[current->fake_len - 1] != '/' &&
+              (uri[current->fake_len] == '\0' ||
+               uri[current->fake_len] == '/')))) {
 #ifdef FASCIST_LOGGING
-            fprintf(stderr, "Got it!\n");
+                fprintf(stderr, "Got it!\n");
 #endif
-            return current;
-        } else
+                return current;
+        }
 #ifdef FASCIST_LOGGING
+        else
             fprintf(stderr, "Don't Got it!\n");
-#else
-            ;
 #endif
         current = current->next;
     }
@@ -160,37 +204,17 @@ int translate_uri(request * req)
     char *p;
     int uri_len;
 
-    /* Percent-decode request */
-    if (unescape_uri(req->request_uri, &(req->query_string)) == 0) {
-        log_error_doc(req);
-        fputs("Problem unescaping uri\n", stderr);
-        send_r_bad_request(req);
-        return 0;
-    }
-
-    /* clean pathname */
-    clean_pathname(req->request_uri);
-
     req_urip = req->request_uri;
     if (req_urip[0] != '/') {
-        send_r_not_found(req);
+        send_r_bad_request(req);
         return 0;
     }
 
     uri_len = strlen(req->request_uri);
 
-    current = find_alias(req->request_uri);
+    current = find_alias(req->request_uri, uri_len);
     if (current) {
 
-        /* FIXME: what does the below code do? */
-        /*
-           if (current->fakename[current->fake_len - 1] != '/' &&
-           req->request_uri[current->fake_len] != '/' &&
-           req->request_uri[current->fake_len] != '\0') {
-           break;
-           }
-         */
-
         if (current->type == SCRIPTALIAS) /* Script */
             return init_script_alias(req, current, uri_len);
 
@@ -211,15 +235,6 @@ int translate_uri(request * req)
                    req->request_uri + current->fake_len,
                    uri_len - current->fake_len + 1);
         }
-        /*
-           strcpy(buffer, current->realname);
-           strcat(buffer, &req->request_uri[current->fake_len]);
-         */
-
-        /*
-           sprintf(buffer, "%s%s", current->realname,
-           &req->request_uri[current->fake_len]);
-         */
 
         if (current->type == REDIRECT) { /* Redirect */
             if (req->method == M_POST) { /* POST to non-script */
@@ -227,10 +242,15 @@ int translate_uri(request * req)
                 send_r_bad_request(req);
                 return 0;       /* not a script alias, therefore begin filling in data */
             }
-            send_redirect_temp(req, buffer, "");
+            send_r_moved_temp(req, buffer, "");
             return 0;
         } else {                /* Alias */
             req->pathname = strdup(buffer);
+            if (!req->pathname) {
+                send_r_error(req);
+                WARN("unable to strdup buffer onto req->pathname");
+                return 0;
+            }
             return 1;
         }
     }
@@ -245,7 +265,8 @@ int translate_uri(request * req)
 
         req_urip = req->request_uri + 2;
 
-        p = strchr(req_urip, '/');
+        /* since we have uri_len which is from strlen(req->request_uri) */
+        p = memchr(req_urip, '/', uri_len - 2);
         if (p)
             *p = '\0';
 
@@ -275,11 +296,6 @@ int translate_uri(request * req)
             if (p)
                 memcpy(buffer + l1 + 1 + l2, p, l3 + 1);
         }
-        /*
-           sprintf(buffer, "%s/%s", user_homedir, user_dir);
-           if (p)
-           strcat(buffer, p);
-         */
     } else if (document_root) {
         /* no aliasing, no userdir... */
         int l1, l2, l3;
@@ -319,6 +335,11 @@ int translate_uri(request * req)
      */
 
     req->pathname = strdup(buffer);
+    if (!req->pathname) {
+        WARN("Could not strdup buffer for req->pathname!");
+        send_r_error(req);
+        return 0;
+    }
 
     /* below we support cgis outside of a ScriptAlias */
     if (strcmp(CGI_MIME_TYPE, get_mime_type(buffer)) == 0) { /* cgi */
@@ -330,6 +351,11 @@ int translate_uri(request * req)
         /* FIXME */
         /* script_name could end up as /cgi-bin/bob/extra_path */
         req->script_name = strdup(req->request_uri);
+        if (!req->script_name) {
+            WARN("Could not strdup req->request_uri for req->script_name");
+            send_r_error(req);
+            return 0;
+        }
         if (req->simple)
             req->is_cgi = NPH;
         else
@@ -376,12 +402,11 @@ int init_script_alias(request * req, alias * current1, int uri_len)
         send_r_bad_request(req);
         return 0;
     }
-    strcpy(pathname, current1->realname);
-    strcat(pathname, &req->request_uri[current1->fake_len]);
-    /*
-       sprintf(pathname, "%s%s", current1->realname,
-       &req->request_uri[current1->fake_len]);
-     */
+
+    memcpy(pathname, current1->realname, current1->real_len);
+    memcpy(pathname + current1->real_len,
+           &req->request_uri[current1->fake_len],
+           uri_len - current1->fake_len + 1); /* the +1 copies the NUL */
 #ifdef FASCIST_LOGGING
     log_error_time();
     fprintf(stderr,
@@ -398,6 +423,10 @@ int init_script_alias(request * req, alias * current1, int uri_len)
      (in /cgi-bin/bob, start at the 'b' in bob */
     index = current1->real_len;
 
+    /* go to first and successive '/' and keep checking
+     * if it is a full pathname
+     * on success (stat (not lstat) of file is a *regular file*)
+     */
     do {
         c = pathname[++index];
         if (c == '/') {
@@ -426,6 +455,13 @@ int init_script_alias(request * req, alias * current1, int uri_len)
         }
     } while (c != '\0');
 
+    req->script_name = strdup(req->request_uri);
+    if (!req->script_name) {
+        send_r_error(req);
+        WARN("unable to strdup req->request_uri for req->script_name");
+        return 0;
+    }
+
     if (c == '\0') {
         err = stat(pathname, &statbuf);
         if (err == -1) {
@@ -458,8 +494,19 @@ int init_script_alias(request * req, alias * current1, int uri_len)
         int path_len;
 
         req->path_info = strdup(pathname + index);
+        if (!req->path_info) {
+            send_r_error(req);
+            WARN("unable to strdup pathname + index for req->path_info");
+            return 0;
+        }
         pathname[index] = '\0'; /* strip path_info from path */
         path_len = strlen(req->path_info);
+        /* we need to fix script_name here */
+        /* index points into pathname, which is
+         * realname/cginame/foo
+         * and index points to the '/foo' part
+         */
+        req->script_name[strlen(req->script_name) - path_len] = '\0'; /* zap off the /foo part */
 
         /* now, we have to re-alias the extra path info....
            this sucks.
@@ -478,13 +525,15 @@ int init_script_alias(request * req, alias * current1, int uri_len)
                     return 0;
                 }
 
-                strcpy(buffer, current->realname);
-                strcat(buffer, &req->path_info[current->fake_len]);
-                /*
-                   sprintf(buffer, "%s%s", current->realname,
-                   &req->path_info[current->fake_len]);
-                 */
+                memcpy(buffer, current->realname, current->real_len);
+                strcpy(buffer + current->real_len,
+                       &req->path_info[current->fake_len]);
                 req->path_translated = strdup(buffer);
+                if (!req->path_translated) {
+                    send_r_error(req);
+                    WARN("unable to strdup buffer for req->path_translated");
+                    return 0;
+                }
             }
             current = current->next;
         }
@@ -516,8 +565,8 @@ int init_script_alias(request * req, alias * current1, int uri_len)
 
                 req->path_translated = malloc(l1 + l2 + l3 + 2);
                 if (req->path_translated == NULL) {
-                    log_error_mesg(__FILE__, __LINE__,
-                                   "unable to malloc memory for req->path_translated");
+                    send_r_error(req);
+                    WARN("unable to malloc memory for req->path_translated");
                     return 0;
                 }
                 memcpy(req->path_translated, user_homedir, l1);
@@ -525,16 +574,9 @@ int init_script_alias(request * req, alias * current1, int uri_len)
                 memcpy(req->path_translated + l1 + 1, user_dir, l2 + 1);
                 if (p)
                     memcpy(req->path_translated + l1 + 1 + l2, p, l3 + 1);
-
-                /*
-                   sprintf(buffer, "%s/%s", user_homedir, user_dir);
-                   if (p)
-                   strcat(buffer, p);
-                   req->path_translated = strdup(buffer);
-                 */
             }
         }
-        if (!req->path_translated) {
+        if (!req->path_translated && document_root) {
             /* no userdir, no aliasing... try document root */
             int l1, l2;
             l1 = strlen(document_root);
@@ -542,32 +584,22 @@ int init_script_alias(request * req, alias * current1, int uri_len)
 
             req->path_translated = malloc(l1 + l2 + 1);
             if (req->path_translated == NULL) {
-                log_error_mesg(__FILE__, __LINE__,
-                               "unable to malloc memory for req->path_translated");
+                send_r_error(req);
+                WARN("unable to malloc memory for req->path_translated");
                 return 0;
             }
             memcpy(req->path_translated, document_root, l1);
             memcpy(req->path_translated + l1, req->path_info, l2 + 1);
-            /*
-               sprintf(buffer, "%s%s", document_root,
-               req->path_info);
-               req->path_translated = strdup(buffer);
-             */
         }
     }
 
     req->pathname = strdup(pathname);
+    if (!req->pathname) {
+        send_r_error(req);
+        WARN("unable to strdup pathname for req->pathname");
+        return 0;
+    }
 
-    /* there used to be some ip stuff in here */
-
-    /* We have to strip some stuff from the pathname because we
-       want an http accessible name, not the "real" name.
-       Thus, instead of /usr/lib/cgi-bin/cgiprogram, we get
-       /cgi-bin/cgiprogram
-       Yay!
-     */
-    req->script_name = strdup(req->pathname + current1->real_len
-                              - current1->fake_len);
     return 1;
 }
 
