@@ -1,6 +1,6 @@
 /*
  *  Boa, an http server
- *  Copyright (C) 1995 Paul Phillips <psp@well.com>
+ *  Copyright (C) 1995 Paul Phillips <paulp@go2net.com>
  *  Some changes Copyright (C) 1996-99 Larry Doolittle <ldoolitt@jlab.org>
  *  Some changes Copyright (C) 1997-99 Jon Nelson <jnelson@boa.org>
  *
@@ -20,16 +20,11 @@
  *
  */
 
-/* $Id: boa.h,v 1.47 2001/10/20 02:52:42 jnelson Exp $*/
+/* $Id: boa.h,v 1.63.2.2 2002/07/26 03:03:44 jnelson Exp $*/
 
 #ifndef _BOA_H
 #define _BOA_H
 
-#include "config.h"
-/* config.h should be 1st include because this sets lots of #defines that
- are used in other header files
- */
-
 #include <errno.h>
 #include <stdlib.h>             /* malloc, free, etc. */
 #include <stdio.h>              /* stdin, stdout, stderr */
@@ -37,15 +32,16 @@
 #include <ctype.h>
 #include <time.h>               /* localtime, time */
 #include <pwd.h>
+#include <grp.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <limits.h>             /* OPEN_MAX */
+#include <setjmp.h>
 
 #include <netdb.h>
 #include <netinet/in.h>
 
 #include <sys/mman.h>
-#include <sys/socket.h>
 #include <sys/select.h>
 #include <sys/types.h>          /* socket, bind, accept */
 #include <sys/socket.h>         /* socket, bind, accept, setsockopt, */
@@ -56,7 +52,6 @@
 #include "globals.h"
 
 /* alias */
-
 void add_alias(char *fakename, char *realname, int script);
 int translate_uri(request * req);
 int init_script_alias(request * req, alias * current, int uri_len);
@@ -75,11 +70,12 @@ int process_get(request * req);
 int get_dir(request * req, struct stat *statbuf);
 
 /* hash */
-int get_mime_hash_value(char *extension);
+unsigned get_mime_hash_value(char *extension);
 char *get_mime_type(char *filename);
 char *get_home_dir(char *name);
 void dump_mime(void);
 void dump_passwd(void);
+void show_hash_stats(void);
 
 /* log */
 void open_logs(void);
@@ -104,11 +100,11 @@ int write_body(request * req);
 /* request */
 request *new_request(void);
 void get_request(int);
-void process_requests(void);
+void process_requests(int server_s);
 int process_header_end(request * req);
 int process_header_line(request * req);
 int process_logline(request * req);
-void process_option_line(request * req);
+int process_option_line(request * req);
 void add_accept_header(request * req, char *mime_type);
 void free_requests(void);
 
@@ -120,8 +116,8 @@ void print_last_modified(request * req);
 void print_http_headers(request * req);
 
 void send_r_request_ok(request * req); /* 200 */
-void send_redirect_perm(request * req, char *url); /* 301 */
-void send_redirect_temp(request * req, char *url, char *more_hdr); /* 302 */
+void send_r_moved_perm(request * req, char *url); /* 301 */
+void send_r_moved_temp(request * req, char *url, char *more_hdr); /* 302 */
 void send_r_not_modified(request * req); /* 304 */
 void send_r_bad_request(request * req); /* 400 */
 void send_r_unauthorized(request * req, char *name); /* 401 */
@@ -129,15 +125,15 @@ void send_r_forbidden(request * req); /* 403 */
 void send_r_not_found(request * req); /* 404 */
 void send_r_error(request * req); /* 500 */
 void send_r_not_implemented(request * req); /* 501 */
+void send_r_bad_gateway(request * req); /* 502 */
+void send_r_service_unavailable(request * req); /* 503 */
 void send_r_bad_version(request * req); /* 505 */
 
 /* cgi */
 void create_common_env(void);
-void create_env(request * req);
-#define env_gen(x,y) env_gen_extra(x,y,0)
-char *env_gen_extra(const char *key, const char *value, int extra);
-void add_cgi_env(request * req, char *key, char *value);
-void complete_env(request * req);
+void clear_common_env(void);
+int add_cgi_env(request * req, char *key, char *value, int http_prefix);
+int complete_env(request * req);
 void create_argv(request * req, char **aargv);
 int init_cgi(request * req);
 
@@ -145,21 +141,29 @@ int init_cgi(request * req);
 void init_signals(void);
 void sighup_run(void);
 void sigchld_run(void);
-void lame_duck_mode_run(int);
+void sigalrm_run(void);
+void sigterm_stage1_run(int);
+void sigterm_stage2_run();
 
 /* util.c */
 void clean_pathname(char *pathname);
 char *get_commonlog_time(void);
 void rfc822_time_buf(char *buf, time_t s);
-char *simple_itoa(int i);
+char *simple_itoa(unsigned int i);
+int boa_atoi(char *s);
 char *escape_string(char *inp, char *buf);
 int month2int(char *month);
 int modified_since(time_t * mtime, char *if_modified_since);
 char *to_upper(char *str);
 int unescape_uri(char *uri, char **query_string);
+int create_temporary_file(short want_unlink, char *storage, int size);
+char * normalize_path(char *path);
+int real_set_block_fd(int fd);
+int real_set_nonblock_fd(int fd);
 
 /* buffer */
 int req_write(request * req, char *msg);
+void reset_output_buffer(request *req);
 int req_write_escape_http(request * req, char *msg);
 int req_write_escape_html(request * req, char *msg);
 int req_flush(request * req);
@@ -181,8 +185,11 @@ int read_from_pipe(request * req);
 int write_from_pipe(request * req);
 
 /* ip */
-int bind_server(int server_s, char *server_ip);
+int bind_server(int server_s, char *ip);
 char *ascii_sockaddr(struct SOCKADDR *s, char *dest, int len);
 int net_port(struct SOCKADDR *s);
 
+/* select */
+void select_loop(int server_s);
+
 #endif
