@@ -1,6 +1,6 @@
 /*
  *  Boa, an http server
- *  Copyright (C) 1995 Paul Phillips <psp@well.com>
+ *  Copyright (C) 1995 Paul Phillips <paulp@go2net.com>
  *  Some changes Copyright (C) 1999 Jon Nelson <jnelson@boa.org>
 
  *  This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
  *
  */
 
-/* $Id: buffer.c,v 1.9 2001/10/20 02:56:36 jnelson Exp $ */
+/* $Id: buffer.c,v 1.10.2.2 2002/07/26 03:03:44 jnelson Exp $ */
 
 #include "boa.h"
 #include "escape.h"
@@ -54,6 +54,11 @@ int req_write(request * req, char *msg)
     return req->buffer_end;
 }
 
+void reset_output_buffer(request *req)
+{
+    req->buffer_end = 0;
+}
+
 /*
  * Name: req_write_escape_http
  * Description: Buffers and "escapes" data before sending to client.
@@ -234,8 +239,11 @@ char *escape_string(char *inp, char *buf)
     if (buf == NULL && max)
         buf = malloc(sizeof (char) * max + 1);
 
-    if (buf == NULL)
+    if (buf == NULL) {
+        log_error_time();
+        perror("malloc");
         return NULL;
+    }
 
     index = buf;
     while ((c = *inp++) && max > 0) {
