@@ -1,6 +1,6 @@
 /*
  *  Boa, an http server
- *  Copyright (C) 1995 Paul Phillips <psp@well.com>
+ *  Copyright (C) 1995 Paul Phillips <paulp@go2net.com>
  *  Some changes Copyright (C) 1997 Jon Nelson <jnelson@boa.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
  *
  */
 
-/* $Id: defines.h,v 1.90 2001/11/02 04:37:46 jnelson Exp $*/
+/* $Id: defines.h,v 1.107.2.2 2002/06/07 02:57:23 jnelson Exp $*/
 
 #ifndef _DEFINES_H
 #define _DEFINES_H
@@ -30,6 +30,12 @@
 #define SERVER_ROOT "/etc/boa"
 #endif
 
+/***** Change this via the CGIPath configuration value in boa.conf *****/
+#define DEFAULT_PATH     "/bin:/usr/bin:/usr/local/bin"
+
+/***** Change this via the SinglePostLimit configuration value in boa.conf *****/
+#define SINGLE_POST_LIMIT_DEFAULT               1024 * 1024 /* 1 MB */
+
 /***** Various stuff that you may want to tweak, but probably shouldn't *****/
 
 #define SOCKETBUF_SIZE				8192
@@ -43,8 +49,7 @@
 
 #define REQUEST_TIMEOUT				60
 
-#define CGI_MIME_TYPE    "application/x-httpd-cgi"
-#define DEFAULT_PATH     "/bin:/usr/bin:/usr/local/bin"
+#define CGI_MIME_TYPE                           "application/x-httpd-cgi"
 
 /***** CHANGE ANYTHING BELOW THIS LINE AT YOUR OWN PERIL *****/
 /***** You will probably introduce buffer overruns unless you know
@@ -62,11 +67,11 @@
 #endif
 
 #ifndef SERVER_VERSION
-#define SERVER_VERSION 				"Boa/0.94.11"
+#define SERVER_VERSION 				"Boa/0.94.13"
 #endif
 
 #define CGI_VERSION				"CGI/1.1"
-#define COMMON_CGI_COUNT 8
+#define COMMON_CGI_COUNT 6
 #define CGI_ENV_MAX     50
 #define CGI_ARGC_MAX 128
 
@@ -120,24 +125,6 @@
 #define M_LINK		6
 #define M_UNLINK	7
 
-/******************* ERRORS *****************/
-
-#define SERVER_ERROR		1
-#define OUT_OF_MEMORY		2
-#define NO_CREATE_SOCKET	3
-#define NO_FCNTL		4
-#define NO_SETSOCKOPT		5
-#define NO_BIND			6
-#define NO_LISTEN		7
-#define NO_SETGID		8
-#define NO_SETUID		9
-#define NO_OPEN_LOG		10
-#define SELECT			11
-#define GETPWUID		12
-#define INITGROUPS		13
-
-#define SHUTDOWN		15            /* do not change */
-
 /************** REQUEST STATUS (req->status) ***************/
 
 #define READ_HEADER             0
@@ -152,9 +139,6 @@
 #define DONE			9
 #define DEAD                   10
 
-#define CLIENT_WRITABLE(status) (status==WRITE || status==PIPE_WRITE)
-#define CLIENT_READABLE(status) (status < BODY_WRITE)
-
 /************** CGI TYPE (req->is_cgi) ******************/
 
 #define CGI                     1
@@ -172,8 +156,6 @@
 #define KA_STOPPED     	1
 #define KA_ACTIVE      	2
 
-#define SQUASH_KA(req)	(req->keepalive=KA_STOPPED)
-
 /********* CGI STATUS CONSTANTS (req->cgi_status) *******/
 #define CGI_PARSE 1
 #define CGI_BUFFER 2
@@ -188,4 +170,29 @@
 
 #define MAX_FILE_MMAP 100 * 1024 /* 100K */
 
+/***************** USEFUL MACROS ************************/
+
+#define SQUASH_KA(req)	(req->keepalive=KA_STOPPED)
+
+#define BOA_FD_SET(fd, where) { FD_SET(fd, where); \
+    if (fd > max_fd) max_fd = fd; \
+    }
+
+/* If and when everyone has a modern gcc or other near-C99 compiler,
+ * change these to static inline functions. Also note that since
+ * we never fuss with O_APPEND append or O_ASYNC, we shouldn't have
+ * to perform an extra system call to F_GETFL first.
+ */
+
+#ifdef BOA_USE_GETFL
+#define set_block_fd(fd)    real_set_block_fd(fd)
+#define set_nonblock_fd(fd) real_set_nonblock_fd(fd)
+#else
+#define set_block_fd(fd)    fcntl(fd, F_SETFL, 0)
+#define set_nonblock_fd(fd) fcntl(fd, F_SETFL, NOBLOCK)
+#endif
+
+#define DIE(mesg) log_error_mesg(__FILE__, __LINE__, mesg), exit(1)
+#define WARN(mesg) log_error_mesg(__FILE__, __LINE__, mesg)
+
 #endif
