@@ -1,7 +1,7 @@
 /*
  *  Boa, an http server
- *  Copyright (C) 1995 Paul Phillips <psp@well.com>
- *  Some changes Copyright (C) 1996 Larry Doolittle <ldoolitt@jlab.org>
+ *  Copyright (C) 1995 Paul Phillips <paulp@go2net.com>
+ *  Some changes Copyright (C) 1996 Larry Doolittle <ldoolitt@boa.org>
  *  Some changes Copyright (C) 1999 Jon Nelson <jnelson@boa.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
  *
  */
 
-/* $Id: log.c,v 1.34 2001/06/30 03:41:08 jnelson Exp $*/
+/* $Id: log.c,v 1.36.2.3 2002/07/26 03:04:48 jnelson Exp $*/
 
 #include "boa.h"
 
@@ -31,6 +31,8 @@ char *access_log_name;
 char *cgi_log_name;
 int cgi_log_fd;
 
+FILE *fopen_gen_fd(char *spec, const char *mode);
+
 FILE *fopen_gen_fd(char *spec, const char *mode)
 {
     int fd;
@@ -45,14 +47,39 @@ FILE *fopen_gen_fd(char *spec, const char *mode)
 /*
  * Name: open_logs
  *
- * Description: Opens up the error log, ties it to stderr, and line
- * buffers it.
+ * Description: Opens access log, error log, and if specified, cgi log
+ * Ties stderr to error log, except during cgi execution, at which
+ * time cgi log is the stderr for cgis.
+ *
+ * Access log is line buffered, error log is not buffered.
+ *
  */
 
 void open_logs(void)
 {
     int error_log;
 
+    /* if error_log_name is set, dup2 stderr to it */
+    /* otherwise, leave stderr alone */
+    /* we don't want to tie stderr to /dev/null */
+    if (error_log_name) {
+        /* open the log file */
+        if (!(error_log = open_gen_fd(error_log_name))) {
+            DIE("unable to open error log");
+        }
+
+        /* redirect stderr to error_log */
+        if (dup2(error_log, STDERR_FILENO) == -1) {
+            DIE("unable to dup2 the error log");
+        }
+        close(error_log);
+    }
+
+    /* set the close-on-exec to true */
+    if (fcntl(STDERR_FILENO, F_SETFD, 1) == -1) {
+        DIE("unable to fcntl the error log");
+    }
+
     if (access_log_name) {
         /* Used the "a" flag with fopen, but fopen_gen_fd builds that in
          * implicitly when used as a file, and "a" is incompatible with
@@ -77,14 +104,13 @@ void open_logs(void)
     if (cgi_log_name) {
         cgi_log_fd = open_gen_fd(cgi_log_name);
         if (cgi_log_fd == -1) {
-            log_error_mesg(__FILE__, __LINE__, "open cgi_log");
+            WARN("open cgi_log");
             free(cgi_log_name);
             cgi_log_name = NULL;
             cgi_log_fd = 0;
         } else {
             if (fcntl(cgi_log_fd, F_SETFD, 1) == -1) {
-                log_error_mesg(__FILE__, __LINE__,
-                               "unable to set close-on-exec flag for cgi_log");
+                WARN("unable to set close-on-exec flag for cgi_log");
                 close(cgi_log_fd);
                 cgi_log_fd = 0;
                 free(cgi_log_name);
@@ -92,31 +118,6 @@ void open_logs(void)
             }
         }
     }
-
-    /* if error_log_name is set, dup2 stderr to it */
-    /* otherwise, leave stderr alone */
-    if (error_log_name) {
-        /* open the log file */
-        if (!(error_log = open_gen_fd(error_log_name))) {
-            log_error_mesg(__FILE__, __LINE__, "unable to open error log");
-            exit(errno);
-        }
-
-        /* redirect stderr to error_log */
-        if (dup2(error_log, STDERR_FILENO) == -1) {
-            log_error_mesg(__FILE__, __LINE__,
-                           "unable to dup2 the error log");
-            exit(errno);
-        }
-        close(error_log);
-    }
-
-    /* set the close-on-exec to true */
-    if (fcntl(STDERR_FILENO, F_SETFD, 1) == -1) {
-        log_error_mesg(__FILE__, __LINE__,
-                       "unable to fcntl the error log");
-        exit(errno);
-    }
 }
 
 /*
